#!/usr/bin/env python
# -*- coding: utf-8 -*-

# py_tools_ds - A collection of geospatial data analysis tools that simplify standard
# operations when handling geospatial raster and vector data as well as projections.
#
# Copyright (C) 2016-2021
# - Daniel Scheffler (GFZ Potsdam, daniel.scheffler@gfz-potsdam.de)
# - Helmholtz Centre Potsdam - GFZ German Research Centre for Geosciences Potsdam,
#   Germany (https://www.gfz-potsdam.de/)
#
# This software was developed within the context of the GeoMultiSens project funded
# by the German Federal Ministry of Education and Research
# (project grant code: 01 IS 14 010 A-C).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


"""
test_map_info
-------------

Tests for `py_tools_ds.geo.map_info` module.
"""

import unittest

from py_tools_ds.geo.map_info import geotransform2mapinfo, mapinfo2geotransform
from py_tools_ds.geo.projection import EPSG2WKT


geotransform_utm_NE = (331185.0, 30.0, -0.0, 5840115.0, -0.0, -30.0)
geotransform_utm_SE = (331185.0, 30.0, -0.0, -5840115.0, -0.0, -30.0)
geotransform_utm_NE_rotated = (331185.0, 12.202099292274006, 27.406363729278027,
                               5840115.0, 27.406363729278027, -12.202099292274006)
geotransform_local = (0, 1, 0, 0, 0, -1)
geotransform_local_rotated = (0.0, 6.123233995736766e-17, 1.0, 0.0, 1.0, -6.123233995736766e-17)
map_info_utm_NE = ['UTM', 1, 1, 331185.0, 5840115.0, 30.0, 30.0, 33, 'North', 'WGS-84']
map_info_utm_SE = ['UTM', 1, 1, 331185.0, -5840115.0, 30.0, 30.0, 33, 'South', 'WGS-84']
map_info_utm_NE_rotated = ['UTM', 1, 1, 331185.0, 5840115.0, 30.0, 30.0, 33, 'North', 'WGS-84', 'rotation=66.00000']
map_info_local = ['Arbitrary', 1, 1, 0, 0, 1, 1, 0, 'North']
map_info_local_rotated = ['Arbitrary', 1, 1, 0, 0, 1, 1, 0, 'North', 'rotation=90.00000']

wkt_utm = \
    """
    PROJCS["WGS 84 / UTM zone 33N",
           GEOGCS["WGS 84",
                  DATUM["WGS_1984",
                        SPHEROID["WGS 84", 6378137, 298.257223563,
                                 AUTHORITY["EPSG", "7030"]],
                        AUTHORITY["EPSG", "6326"]],
                  PRIMEM["Greenwich", 0,
                         AUTHORITY["EPSG", "8901"]],
                  UNIT["degree", 0.0174532925199433,
                       AUTHORITY["EPSG", "9122"]],
                  AUTHORITY["EPSG", "4326"]],
           PROJECTION["Transverse_Mercator"],
           PARAMETER["latitude_of_origin", 0],
           PARAMETER["central_meridian", 15],
           PARAMETER["scale_factor", 0.9996],
           PARAMETER["false_easting", 500000],
           PARAMETER["false_northing", 0],
           UNIT["metre", 1,
                AUTHORITY["EPSG", "9001"]],
           AXIS["Easting", EAST],
           AXIS["Northing", NORTH],
           AUTHORITY["EPSG", "32633"]]
    """
wkt_utm = ' '.join(wkt_utm.split())


class Test_geotransform2mapinfo(unittest.TestCase):

    # TODO implement test in case of geographic prj

    def test_UTM_gt_prj_NE(self):
        map_info = geotransform2mapinfo(gt=geotransform_utm_NE, prj=wkt_utm)
        self.assertTrue(isinstance(map_info, list))
        self.assertEqual(map_info, map_info_utm_NE)

    def test_UTM_gt_prj_NW(self):
        gt = (394185.0, 30.0, -0.0, 4572315.0, -0.0, -30.0)
        prj = EPSG2WKT(32610)  # UTM zone 10 North
        exp = ['UTM', 1.0, 1.0, 394185.0, 4572315.0, 30.0, 30.0, 10, 'North', 'WGS-84']

        map_info = geotransform2mapinfo(gt=gt, prj=prj)
        self.assertTrue(isinstance(map_info, list))
        self.assertEqual(map_info, exp)

    def test_UTM_gt_prj_SW(self):
        # option 1: projection says 'South', gt has a positive y origin (e.g., Sentinel-2 data)
        gt = (394185.0, 30.0, -0.0, 4572315.0, -0.0, -30.0)
        prj = EPSG2WKT(32710)  # UTM zone 10 South
        exp = ['UTM', 1.0, 1.0, 394185.0, 4572315.0, 30.0, 30.0, 10, 'South', 'WGS-84']

        map_info = geotransform2mapinfo(gt=gt, prj=prj)
        self.assertTrue(isinstance(map_info, list))
        self.assertEqual(map_info, exp)

        # option 2: projection says 'North', gt has a negative y origin (e.g., Landsat-5 data)
        gt = (394185.0, 30.0, -0.0, -4572315.0, -0.0, -30.0)
        prj = EPSG2WKT(32610)  # UTM zone 10 North
        exp = ['UTM', 1.0, 1.0, 394185.0, -4572315.0, 30.0, 30.0, 10, 'North', 'WGS-84']

        map_info = geotransform2mapinfo(gt=gt, prj=prj)
        self.assertTrue(isinstance(map_info, list))
        self.assertEqual(map_info, exp)

    def test_UTM_gt_prj_SE(self):
        prj = EPSG2WKT(32733)  # UTM zone 33 South

        map_info = geotransform2mapinfo(gt=geotransform_utm_SE, prj=prj)
        self.assertTrue(isinstance(map_info, list))
        self.assertEqual(map_info, map_info_utm_SE)

    def test_gt_is_none(self):
        # test gt=None
        map_info = geotransform2mapinfo(gt=None, prj=wkt_utm)
        self.assertTrue(isinstance(map_info, list))
        self.assertEqual(map_info, map_info_local)

    def test_gt_is_arbitrary(self):
        # test gt=[0, 1, 0, 0, 0, -1]
        map_info = geotransform2mapinfo(gt=geotransform_local, prj=wkt_utm)
        self.assertTrue(isinstance(map_info, list))
        self.assertEqual(map_info, map_info_local)

    def test_prj_is_empty(self):
        exp_map_info = ['Arbitrary', 1, 1, 331185.0, 5840115.0, 30.0, 30.0, 0, 'North']
        map_info = geotransform2mapinfo(gt=geotransform_utm_NE, prj='')
        self.assertTrue(isinstance(map_info, list))
        self.assertEqual(map_info, exp_map_info)

    def test_prj_is_not_wgs84(self):
        exp_map_info = ['Lambert Azimuthal Equal Area', 1.0, 1.0, 4526026.0, 3284919.5, 10.0, 10.0]
        map_info = geotransform2mapinfo(gt=(4526026.0, 10.0, 0.0, 3284919.5, 0.0, -10.0), prj=EPSG2WKT(3035))
        self.assertTrue(isinstance(map_info, list))
        self.assertEqual(map_info, exp_map_info)

    def test_prj_is_not_wgs84_rotation(self):
        exp_map_info = ['Lambert Azimuthal Equal Area', 1.0, 1.0, 331185.0, 5840115.0, 30.0, 30.0, 'rotation=66']
        map_info = geotransform2mapinfo(gt=geotransform_utm_NE_rotated, prj=EPSG2WKT(3035))
        self.assertTrue(isinstance(map_info, list))
        self.assertEqual(map_info, exp_map_info)

    def test_gt_contains_rotation(self):
        map_info = geotransform2mapinfo(gt=geotransform_utm_NE_rotated, prj=wkt_utm)
        self.assertTrue(isinstance(map_info, list))
        self.assertEqual(map_info, map_info_utm_NE_rotated)

    def test_gt_contains_rotation_prj_is_local(self):
        map_info = geotransform2mapinfo(gt=geotransform_local_rotated, prj='')
        self.assertTrue(isinstance(map_info, list))
        self.assertEqual(map_info, map_info_local_rotated)


class Test_mapinfo2geotransform(unittest.TestCase):

    def test_map_info_is_valid(self):
        gt = mapinfo2geotransform(map_info_utm_NE)
        self.assertTrue(isinstance(gt, (tuple, list)))
        self.assertEqual(gt, geotransform_utm_NE)

        gt = mapinfo2geotransform(map_info_utm_SE)
        self.assertTrue(isinstance(gt, (tuple, list)))
        self.assertEqual(gt, geotransform_utm_SE)

        # test arbitrary map info
        gt = mapinfo2geotransform(['Arbitrary', 1, 1, 5, -7, 1, 1, 0, 'North'])
        self.assertTrue(isinstance(gt, (tuple, list)))
        self.assertEqual(gt, (5, 1, 0, -7, 0, -1))

    def test_map_info_is_empty(self):
        gt = mapinfo2geotransform(None)
        self.assertTrue(isinstance(gt, (tuple, list)))
        self.assertEqual(gt, geotransform_local)

    def test_map_info_contains_LAEA_proj(self):
        gt = mapinfo2geotransform(['Lambert Azimuthal Equal Area', 1.0, 1.0, 4526026.0, 3284919.5, 10.0, 10.0])
        self.assertTrue(isinstance(gt, (tuple, list)))
        self.assertEqual(gt, (4526026.0, 10.0, 0.0, 3284919.5, 0.0, -10.0))

    def test_map_info_contains_rotation(self):
        gt = mapinfo2geotransform(map_info_utm_NE_rotated)
        self.assertTrue(isinstance(gt, (tuple, list)))
        self.assertEqual(gt, geotransform_utm_NE_rotated)

    def test_map_info_is_local_contains_rotation(self):
        gt = mapinfo2geotransform(map_info_local_rotated)
        self.assertTrue(isinstance(gt, (tuple, list)))
        self.assertEqual(gt, geotransform_local_rotated)


if __name__ == '__main__':
    unittest.main()
