#!/usr/bin/env python
# -*- coding: utf-8 -*-

# py_tools_ds - A collection of geospatial data analysis tools that simplify standard
# operations when handling geospatial raster and vector data as well as projections.
#
# Copyright (C) 2016-2021
# - Daniel Scheffler (GFZ Potsdam, daniel.scheffler@gfz-potsdam.de)
# - Helmholtz Centre Potsdam - GFZ German Research Centre for Geosciences Potsdam,
#   Germany (https://www.gfz-potsdam.de/)
#
# This software was developed within the context of the GeoMultiSens project funded
# by the German Federal Ministry of Education and Research
# (project grant code: 01 IS 14 010 A-C).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
test_topology
-------------

Tests for `py_tools_ds.geo.vector.topology` module.
"""

from unittest import TestCase, main
from shapely.geometry import Polygon, MultiPolygon, Point

from py_tools_ds.geo.vector.topology import get_overlap_polygon, fill_holes_within_poly


class Test_get_overlap_polygon(TestCase):
    def setUp(self):
        self.polygon_1 = Polygon([(0, 0), (0, 5), (5, 5), (5, 0)])
        self.polygon_2 = Polygon([(1, 0), (1, 5), (6, 5), (6, 0)])
        self.polygon_3 = Polygon([(-1, 0), (-1, -5), (-6, -5), (-6, 0)])

    def test_with_overlap(self):
        out = get_overlap_polygon(self.polygon_1, self.polygon_2)
        self.assertIsInstance(out, dict)

        self.assertTrue(out['overlap poly'].equals(Polygon([(1, 0), (1, 5), (5, 5), (5, 0)])))
        self.assertEqual(out['overlap percentage'], 80)
        self.assertEqual(out['overlap area'], 20)

    def test_no_overlap(self):
        out = get_overlap_polygon(self.polygon_1, self.polygon_3)
        self.assertIsInstance(out, dict)

        self.assertTrue(out['overlap poly'] is None)
        self.assertEqual(out['overlap percentage'], 0)
        self.assertEqual(out['overlap area'], 0)

    # TODO test the case where the output polygon is a multipolygon


class Test_fill_holes_within_poly(TestCase):
    def setUp(self):
        self.polygon_1 = Polygon([(0, 0), (5, 0), (5, 5), (0, 5)])  # large poly
        self.polygon_2 = Polygon([(1, 1), (1, 2), (2, 2), (2, 1)])  # small poly within large poly
        self.polygon_3 = Polygon([(3, 3), (3, 4), (5, 4), (4, 3)])  # small poly within large poly
        self.multipolygon_1 = MultiPolygon([self.polygon_1, self.polygon_2])
        self.multipolygon_2 = MultiPolygon([self.polygon_2, self.polygon_3])

    def test_polygon(self):
        filled_poly = fill_holes_within_poly(self.polygon_1)
        self.assertIsInstance(filled_poly, Polygon)
        self.assertTrue(filled_poly == self.polygon_1)

    def test_multipolygon_with_smaller_poly_inside(self):
        filled_poly = fill_holes_within_poly(self.multipolygon_1)
        self.assertIsInstance(filled_poly, Polygon)
        self.assertTrue(filled_poly == self.polygon_1)

    def test_multipolygon_is_disjunct(self):
        with self.assertWarns(RuntimeWarning):
            filled_poly = fill_holes_within_poly(self.multipolygon_2)

        self.assertIsInstance(filled_poly, Polygon)
        self.assertTrue(filled_poly == self.polygon_3)

    def test_empty_multipolygon(self):
        with self.assertRaises(ValueError):
            fill_holes_within_poly(MultiPolygon())

    def test_wrong_input(self):
        with self.assertRaises(ValueError):
            # noinspection PyTypeChecker
            fill_holes_within_poly(Point((0, 1)))


if __name__ == '__main__':
    main()
